@@ -6,7 +6,6 @@ Advanced Interface Switcher
 Abstract interface object
 '''
 
-
 from __future__ import with_statement
 import gtk, hildon, dbus, threading
 from advpowcommon.ui.Banners import HildonBanner
@@ -35,56 +34,59 @@ class AbstractInterface():
     self.__show_settings_dialog = cb_show_settings_dialog
     self.__interfaces = {}
     self.__lcl = locale.get()
+    self.__config = config.get()
+    self.__button = None
     self.interfaces = {}
     self.network = None
     self.state = {}
-    self.__config = config.get()
+
     self.force_startup_mode = False
-    self.pre_init()
+    self.init_state()
     self.__name = self.state.pop('name') if 'name' in self.state else 'IFace'
     self.__config_path = 'settings/%s' % (self.__name.lower())
-    self.__button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, 
-                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
-    self.__button.set_image(gtk.Image())
-    #self.__button.set_image_position(gtk.POS_LEFT|gtk.POS_RIGHT)
-    self.__button.set_alignment(0.5,0.5,1,1)
-    #self.__button.set_image_alignment(0.5,0.5)
-    #self.__button.set_title_alignment(0,0.5)
-    #self.__button.set_value_alignment(0,0.5)
-    self.__button.connect('clicked', self.__button_clicked)
-    self.__button.tap_and_hold_setup(None, None, 0)
-    self.__button.connect('tap-and-hold', self.__button_tap_and_hold)
-    self.__button.connect('destroy', self.__destroy)
-    self.__switch_dialog = None
     self.__smi_cacher = cacher.get()
-    self.__smi_animator = DITPBCachedAnimator([],
-                                              self.__button.get_image().set_from_pixbuf,
-                                              cacher=self.__smi_cacher,
-                                              time_out=250)
+    self.__create_button()
+    self.__switch_dialog = None
     self.state_control = None
     self.mode_control = None
-
     self.set_config_option('mode-on-startup', self.get_modes())
-
-    self.start_animation()
-    self.__init_interfaces()
-    self.__get_status()
-    self.__init_signals()
-    self.stop_animation()
-
-    self.post_init()
-
+    self.__startup()
     start_state = int(self.get_config_value('enable-on-startup'))
     self.__switch_state(start_state)
     if start_state or self.force_startup_mode:
       self.__switch_mode(self.get_modes().index(self.get_config_value('mode-on-startup')), start_state == InterfaceState.off)
-
     self.__running = True
     #print self.__name, self.state['value'], self.state['mode'], self.state['strength']
-    #gobject.timeout_add(10000, self.__show_switch_dialog)
+    #gobject.timeout_add(5000, self.__show_switch_dialog)
     #self.__show_switch_dialog()
 
-  def pre_init(self):
+  def is_running(self):
+    with self.__processing:
+      return self.__running
+
+  def __startup(self):
+    self.__init_interfaces()
+    self.__get_status()
+    self.__init_signals()
+    self.post_init()
+
+  def __create_button(self):
+    with self.__processing:
+      if not self.__button:
+        self.__button = hildon.Button(
+          gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, 
+          hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        self.__button.set_image(gtk.Image())
+        self.__button.set_alignment(0.5,0.5,1,1)
+        self.__button.set_image_alignment(0.5,0.5)
+        self.__button.set_title_alignment(0,0.5)
+        self.__button.set_value_alignment(0,0.5)
+        self.__button.connect('clicked', self.__button_clicked)
+        self.__button.tap_and_hold_setup(None, None, 0)
+        self.__button.connect('tap-and-hold', self.__button_tap_and_hold)
+        self.__button.connect('destroy', self.__destroy)
+
+  def init_state(self):
     pass
 
   def post_init(self):
@@ -120,17 +122,22 @@ class AbstractInterface():
   def set_config_value(self, key, value, save=False):
     return self.__config.set_value('%s/%s' % (self.__config_path, key), value, save)
 
-  def destroy(self):
-    self.__destroy()
+  def destroy(self, button=None):
+    with self.__processing:
+      if button or not self.__button:
+        self.__destroy()
+      else:
+        self.__button.destroy()
 
   def __destroy(self, button=None):
     if hasattr(self, 'state'):
       if self.__switch_dialog:
         self.__switch_dialog.destroy()
+        self.__switch_dialog = None
       self.__destroy_signals()
       self.__destroy_interfaces()
-      #print '%s (%s): destroyed' % (self.__name, self.__class__)
-    self.__running = False
+      self.__button = None
+      self.__set_status_area_icon(self.__name, None, False)
 
   def __init_signals(self):
     for key in self.state['dbus']:
@@ -146,7 +153,6 @@ class AbstractInterface():
           self.state['dbus'][x]['match'] = self.__bus\
             .add_signal_receiver(getattr(self, 'received_%s' % x),
                                  **self.state['dbus'][x])
-          #print '%s | signal \'%s\' created' % (self.__name, x)
           return True
     except:
       print '%s | signal \'%s\' failed' % (self.__name, x)
@@ -167,7 +173,6 @@ class AbstractInterface():
       self.__bus.remove_signal_receiver(receiver_match,**self.state['dbus'][x])
       receiver_match.remove()
       delattr(self, 'received_%s' % x)
-      #print '%s | signal \'%s\' destroyed' % (self.__name, x)
       return True
     return False
 
@@ -201,7 +206,6 @@ class AbstractInterface():
     updated_value = self.__set_value(value)
     updated_mode = self.__set_mode(mode)
     updated_strength = self.__set_strength(strength)
-
     if updated_value or updated_strength:
       self.__update_ui()
     if updated_value or updated_mode:
@@ -210,7 +214,6 @@ class AbstractInterface():
 
   def __init_interfaces(self):
     for x in self.interfaces:
-      #print self.__name, x, 
       self.__init_interface(x)
 
   def __init_interface(self, x, force=False):
@@ -261,13 +264,6 @@ class AbstractInterface():
     self.state['value'], self.state['mode'], self.state['strength'] = self.get_status()
     self.__update_ui()
 
-  def switched(self):
-    self.stop_animation()
-    #try:
-    #  self.__processing.release()
-    #except:
-    #  pass
-
   def update(self):
     self.__update_ui()
 
@@ -277,8 +273,9 @@ class AbstractInterface():
     image = self.state['images_st'][strength] \
       if value == InterfaceState.connected and strength > 0 \
       else self.state['images'][value]
-    self.__set_si(image)
-    self.__button.set_value( \
+    if self.__button:
+      self.__set_si(image)
+      self.__button.set_value( \
         self.network \
         if value == InterfaceState.connected and self.get_config_value('show-network-in-sm') and self.network \
         else self.__lcl.get(self.state['text'][value])
@@ -295,47 +292,33 @@ class AbstractInterface():
     if show_dialog:
       gobject.idle_add(self.__show_switch_dialog)
     else:
-      self.start_animation()
       self.__switch_state(int(self.state['value'] == InterfaceState.off))
     return True
 
   def __show_switch_dialog(self):
-    if not self.__switch_dialog:
-      self.__switch_dialog = gtk.Dialog('%s' % (self.__name), None,#self.__button.window, 
+    if self.__button:
+      if not self.__switch_dialog:
+        self.__switch_dialog = gtk.Dialog('%s' % (self.__name), None,#self.__button.window, 
                                  gtk.DIALOG_NO_SEPARATOR,
                                  (gtk.STOCK_PREFERENCES, gtk.RESPONSE_OK))
-      #gtk.DIALOG_MODAL | 
-      #gtk.DIALOG_DESTROY_WITH_PARENT | 
-      self.state_control = ToggleControlMaemo5(self.__get_state_controls())
-      self.state_control.set_active(self.__get_state_control_value())
-      self.__switch_dialog.vbox.pack_start(self.state_control.get_box(), True)
-      height = self.state_control.get_height()
+        self.state_control = ToggleControlMaemo5(self.__get_state_controls())
+        self.state_control.set_active(self.__get_state_control_value())
+        self.__switch_dialog.vbox.pack_start(self.state_control.get_box(), True)
+        height = self.state_control.get_height()
 
-      if len(self.state['modes']) > 1:
-        self.mode_control = ToggleControlMaemo5(self.__get_mode_controls())
-        self.mode_control.set_active(self.state['mode'])
-        self.__switch_dialog.vbox.pack_start(self.mode_control.get_box(), True)
-        height += self.mode_control.get_height()
+        if len(self.state['modes']) > 1:
+          self.mode_control = ToggleControlMaemo5(self.__get_mode_controls())
+          self.mode_control.set_active(self.state['mode'])
+          self.__switch_dialog.vbox.pack_start(self.mode_control.get_box(), True)
+          height += self.mode_control.get_height()
 
-      settings_button = self.__switch_dialog.action_area.get_children()[0]
-      settings_button.connect('clicked', self.__settings_button_clicked)
+        settings_button = self.__switch_dialog.action_area.get_children()[0]
+        settings_button.connect('clicked', self.__settings_button_clicked)
 
-    #settings_dialog.vbox.set_size_request(-1, min(350, height))
-    self.__switch_dialog.show_all()
-    #self.__processing.release()
-    response = self.__switch_dialog.run()
-    #if response == gtk.RESPONSE_DELETE_EVENT:
-      #self.__switch_dialog.hide()
-      #settings_dialog.destroy()
-      #self.state_control.destroy()
-      #self.state_control = None
-      #if self.mode_control:
-        #self.mode_control.destroy()
-        #self.mode_control = None
-    #settings_dialog.destroy()
-    self.__switch_dialog.hide()
-#    if response == gtk.RESPONSE_OK:
-#      gobject.idle_add(self.__show_switch_dialog)
+      self.__switch_dialog.show_all()
+      response = self.__switch_dialog.run()
+      if self.__switch_dialog:
+        self.__switch_dialog.hide()
     return False
 
   def __settings_button_clicked(self, button=None, user_data=None):
@@ -397,7 +380,7 @@ class AbstractInterface():
           else:
             self.turn_off()
         except:
-          self.__fallback_switched()
+          self.update()
       #else:
       #  self.__processing.release()
 
@@ -410,29 +393,21 @@ class AbstractInterface():
         try:
           self.turn_mode(new_mode, startup)
         except:
-          self.__fallback_switched()
+          self.update()
       #else:
       #  self.__processing.release()
 
-  def __fallback_switched(self):
-    self.switched()
-    self.__update_ui()
-
   def turn_mode(self, mode, startup):
     pass
 
-  def start_animation(self):
-    self.__smi_animator.start()
-
-  def stop_animation(self):
-    self.__smi_animator.stop()
-
   def __set_si(self, key):
     self.__set_smi(key)
     self.__set_sai(key)
 
   def __set_smi(self, key):
-    self.__button.get_image().set_from_pixbuf(self.__smi_cacher.get(key))
+    if self.__button:
+      #print self.__name, key, self.__smi_cacher.get(key) != None
+      self.__button.get_image().set_from_pixbuf(self.__smi_cacher.get(key))
 
   def __set_sai(self, key):
     if any(self.state['show']):
@@ -459,3 +434,14 @@ class AbstractInterface():
     cb = 'config_changed_%s' % key.replace('-','_')
     if hasattr(self, cb):
       getattr(self, cb)(old_value, new_value)
+
+  def config_changed_enabled(self, old_value, new_value):
+    if old_value:
+      if self.__button:
+        self.__button.destroy()
+        self.__button = None
+      else:
+        self.destroy()
+    elif not self.__button:
+      self.__startup()
+      self.__create_button()
