@@ -26,7 +26,7 @@ class GSMInterface(AbstractInterface):
   #status_on = (3,4,5,9,10,11)
   #status_connected = (0,1,2)
 
-  def pre_init(self):
+  def init_state(self):
     self.interfaces = {'Phone': ['com.nokia.phone.net',
                                  '/com/nokia/phone/net',
                                  'Phone.Net',
@@ -133,12 +133,10 @@ class GSMInterface(AbstractInterface):
 
   def run_commands(self, flag):
     self.get_interface('SSC').set_radio(flag)
-    self.switched()
     return False
 
   def run_mode_commands(self, mode):
     self.get_interface('Phone').set_selected_radio_access_technology(dbus.Byte(mode))
-    self.switched()
     return False
 
   def get_mode(self):
