@@ -10,6 +10,7 @@ from __future__ import with_statement
 import dbus, gobject, threading
 from advifswlib.interfaces.Abstract import InterfaceState, AbstractInterface
 from advpowcommon.util.execn import *
+import advifswlib.util.worker as worker
 
 class WLANInterface(AbstractInterface):
 
@@ -61,10 +62,11 @@ class WLANInterface(AbstractInterface):
               'check_be': 'sudo /usr/bin/advifsw/get_wlan_path.sh'
              }
   # off: down > unload
-  wifi_turn_off_cmd = [wifi_cmd['down'], wifi_cmd['unload']]
+  wifi_turn_off_cmd = (wifi_cmd['down'], wifi_cmd['unload'])
   wifi_drivers = ['wl12xx', 'wl1251']
 
-  def pre_init(self):
+  def init_state(self):
+    self.__worker = worker.get()#.prestart()
     self.__lock = threading.Lock()
     self.__strength_timeout = None
     self.__autooff = None
@@ -122,10 +124,9 @@ class WLANInterface(AbstractInterface):
                   'banner': (True, True, True, True, True),
                   'value': InterfaceState.off,
                   'strength': 0,
-                  'modes': ['wlan-stock'],
+                  'modes': ['wlan-stock', 'wlan-bleeding-edge']
+                             if self.__has_be else ['wlan-stock'],
                   'mode': 0}
-    if self.__has_be:
-      self.state['modes'].append('wlan-bleeding-edge')
 
   def post_init(self):
     self.set_config_option('disconnected-timeout', (-1,10,30,60,300))
@@ -165,11 +166,9 @@ class WLANInterface(AbstractInterface):
           if autooff > 0:
             self.__autooff = gobject.timeout_add(autooff*1000, self.turn_off)
       elif args[7] in (1,15) and self.state['value'] != InterfaceState.off:
-        #result = InterfaceState.on
         result = InterfaceState.connecting
         self.__cancel_autooff()
       elif args[7] == 3 and self.state['value'] == InterfaceState.connected:
-        #result = InterfaceState.on
         result = InterfaceState.disconnecting
         self.__cancel_get_strength()
       else:
@@ -195,7 +194,8 @@ class WLANInterface(AbstractInterface):
   def turn_on(self):
     if self.state['value'] == InterfaceState.off:
       #gobject.idle_add(self.run_commands, self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up'])
-      run_async(self.run_commands, [self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up']])
+      #self.run_async(self.run_commands, (self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up']))
+      self.run_commands(self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up'])
     elif self.state['value'] == InterfaceState.connected:
       try:
         self.get_interface('ICD2').disconnect_req(dbus.UInt32(0x8000))
@@ -207,14 +207,23 @@ class WLANInterface(AbstractInterface):
     self.__cancel_autooff()
     if self.state['value'] != InterfaceState.off:
       self.disconnect()
-      run_async(self.run_commands, self.wifi_turn_off_cmd)
+      #self.run_async(self.run_commands, self.wifi_turn_off_cmd)
       #gobject.idle_add(self.run_commands, *self.wifi_turn_off_cmd)
+      self.run_commands(*self.wifi_turn_off_cmd)
     return False
 
+  def set_button_text(self, title, value):
+    self.get_button().set_text(title, value)
+    return False
+
+  def run_async(self, func, args=[], kwargs={}):
+    self.__worker.execute(func, args, kwargs)
+    #keep it warm
+    for i in range(100): pass
+
   def run_commands(self, *cmds):
     with self.__lock:
-      run_shell_command(' && '.join(cmds))
-      self.switched()
+      run_shell_command2(' && '.join(cmds))
     return False
 
   def show_connection_dialog(self):
@@ -252,8 +261,8 @@ class WLANInterface(AbstractInterface):
         self.disconnect()
         # down > unload > load%mode% > up
         #gobject.idle_add(self.run_commands, self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up'])
-        run_async(self.run_commands, [self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up']])
-    self.switched()
+        #self.run_async(self.run_commands, (self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up']))
+        self.run_commands(self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up'])
 
   def __cancel_autooff(self):
     if self.__autooff:
